"""Low-level utilities for both NUA build and NUA agent."""

# Version of module available at first level, __version__
# computed from pyproject data in version.py module
from .version import __version__  # noqa: F401
